import java.io.*; 
import java.util.Iterator;
import java.util.Map.Entry;

/** 
 *  UsaProxy - HTTP proxy for tracking and logging of user interactions on websites
    Copyright (C) 2006  Monika Wnuk - Media Informatics Group at the University of Munich
    <br><br>
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    <br><br>
 *  Class ServerRequest forwards the client request to the server
 *  Thread due to asynchronity of requests and responses
 */
public class ServerRequest extends Thread {

    private SocketData		client,server;
    private ClientRequest	clientRequest;
    
    /**
     *  Constructor, creates a ServerRequest
     *  @param client is the requesting client
     *  @param server is the host which holds the requested resource
     *  @param clientRequest is the thread which manages the request
     */
    public ServerRequest (SocketData client, SocketData server, ClientRequest clientRequest) throws Exception {

		this.client 			= client;
		this.server 			= server;
		this.clientRequest 		= clientRequest;
	
		/** start thread */
		start();
    } 
    
    /**
     *  Sets up a connection to the server and sends request.
     *  <br><br>
     *  The request is stored in a new HTTPTraffic txt-file with a current HttpTraffic index postfix.
     */
    public void run () { 

		try { 
	
			server.setOut(new DataOutputStream(new BufferedOutputStream(server.getOut())));
			StringBuffer headers = new StringBuffer();
			
			for (Iterator e = this.client.getHeaders().entrySet().iterator(); e.hasNext();) {
		
		        Entry me 	= (Entry) e.next();
		        String key 	= (String) me.getKey();
		        if (null == key) continue;
		        if (HTTPData.isRequestLine(key.trim()))
		        	headers.insert(0,key + "\r\n");
		        else {
		        	String value = (String)me.getValue();
		        	headers.append(key).append(": ").append(value).append("\r\n");
		        }
		    }
			headers.append("\r\n");
			
			if (UsaProxy.DEBUG) System.err.println(headers);
			
			server.getOut().write(headers.toString().getBytes(HTTPData.HEADER_DEFAULT_CHAR_ENCODING));
			server.getOut().flush();	

			/** create new httpTraffic index for the request that will be
			 *  used for suffixing the txt-file the request and response data will be stored to
			 *  Additionally, store the request */
			if(clientRequest.getHttpTraffic().isLogHttpTraffic()) {
				clientRequest.getHttpTraffic().store(clientRequest.getHttpTrafficIndex(),"##### Request #####\r\n\r\n"+headers.toString());
			}
			
			/** if any additional data was sent with the request headers (POST) */
			if(client.getIn()!=null) {	
				byte request[] = new byte[4096]; /** data buffer */  
	        	int requestLength;   /** length of request as number of bytes */
	        
	        	while ((requestLength = client.getIn().read(request)) != -1) {
					if (requestLength > 0) {
				/**		for(int gg=0; gg<request.length; gg++)
						System.out.print((char)request[gg]);   */
						server.getOut().write (request,0,requestLength);
						server.getOut().flush();
			        
			        	/** store the request to txt-file */
			        	if(clientRequest.getHttpTraffic().isLogHttpTraffic()) 
							clientRequest.getHttpTraffic().store(clientRequest.getHttpTrafficIndex(), request, 0, requestLength);
			        }
				}
			}
			
			server.getOut().flush();
		    
		} catch (IOException e) {
        	System.err.println("\nAn ERROR occured while sending request: "
							+ e );
		}
    }

}
