import java.net.InetAddress;
import java.net.Socket;
import java.net.URL;

/** Class TransparentRemoteProxy provides methods which are specifically defined for
 *  the case UsaProxy is used as transparent proxy and contacts the web server through
 *  a proxy gateway
 */
public class TransparentRemoteProxy implements ProxyType {
	
	private InetAddress		remoteProxy;
	private int 			remotePort;						  /** server port */
	
	/** Constructor, creates a TransparentRemoteProxy.
     */
	public TransparentRemoteProxy() {
		super();
	}
	
	/** Constructor, creates a TransparentRemoteProxy with the given remote proxy information.
	 * 
	 * @param remoteProxy is the gateway
	 * @param remotePort is the gateway port
     */
	public TransparentRemoteProxy(InetAddress remoteProxy, int remotePort) {
		super();
		this.remotePort 	= remotePort;
		this.remoteProxy 	= remoteProxy;
	}
	
	/**
     *  Returns a Socket of the gateway.
     *  
     *  @return the Socket
     */
	public Socket getServerConnect() throws Exception {
		return new Socket(remoteProxy.getHostName(),remotePort);
	}
	
	/**
     *  Returns the request line.
     *  <br><br>
     *  The URL is left unaltered. HTTP version 1.0 is added.
     *  
     *  @param method is the request method
     *  @param url is the requested URL
     *  @return the request String
     */
	public String getRequestLine(String method, URL url) {
		return method + " " + url.toString() + " HTTP/1.0";
	}
	
	/**
     *  Returns the script String in the form of a script tag with the proxyscript.js as src attribute.
     *  The virtual name lo.lo is used as host.
     *  
     *  @return the script String
     */
	public String getScriptString() {
		return "<script src='http://lo.lo/usaproxylolo/proxyscript.js' type='text/javascript'></script>";
	}

	/** not implemented by TransparentRemoteProxy */
	public void setRequestURL(URL url) {}
	
	/** not implemented by TransparentRemoteProxy */
	public String getHostName() { return ""; }

}
